@@ -7,7 +7,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 1992-2012 Lorenzo Marcantonio, l.marcantonio@logossrl.com
- * Copyright (C) 1992-2023, 2024 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,12 +29,14 @@
 
 #include <algorithm>
 #include <cstdio> // snprintf
+#include <stack>
 
 #include <wx/filename.h>
 #include <wx/mstream.h>
 #include <wx/zstream.h>
 #include <wx/wfstream.h>
 #include <wx/datstrm.h>
+#include <wx/tokenzr.h>
 
 #include <advanced_config.h>
 #include <common.h>               // ResolveUriByEnvVars
@@ -481,9 +483,10 @@ void PDF_PLOTTER::PlotImage( const wxImage& aImage, const VECTOR2I& aPos, double
             if( image.HasAlpha() != aCurrImage.HasAlpha() )
                 continue;
 
-            if( image.HasMask() != aCurrImage.HasMask() || image.GetMaskRed() != aCurrImage.GetMaskRed()
-                || image.GetMaskGreen() != aCurrImage.GetMaskGreen()
-                || image.GetMaskBlue() != aCurrImage.GetMaskBlue() )
+            if( image.HasMask() != aCurrImage.HasMask()
+              || image.GetMaskRed() != aCurrImage.GetMaskRed()
+              || image.GetMaskGreen() != aCurrImage.GetMaskGreen()
+              || image.GetMaskBlue() != aCurrImage.GetMaskBlue() )
                 continue;
 
             int pixCount = image.GetWidth() * image.GetHeight();
@@ -491,7 +494,8 @@ void PDF_PLOTTER::PlotImage( const wxImage& aImage, const VECTOR2I& aPos, double
             if( memcmp( image.GetData(), aCurrImage.GetData(), pixCount * 3 ) != 0 )
                 continue;
 
-            if( image.HasAlpha() && memcmp( image.GetAlpha(), aCurrImage.GetAlpha(), pixCount ) != 0 )
+            if( image.HasAlpha()
+              && memcmp( image.GetAlpha(), aCurrImage.GetAlpha(), pixCount ) != 0 )
                 continue;
 
             return imgHandle;
@@ -655,19 +659,29 @@ void PDF_PLOTTER::closePdfStream()
 }
 
 
-void PDF_PLOTTER::StartPage( const wxString& aPageNumber, const wxString& aPageName )
+void PDF_PLOTTER::StartPage( const wxString& aPageNumber, const wxString& aPageName,
+                             const wxString& aParentPageNumber, const wxString& aParentPageName )
 {
     wxASSERT( m_outputFile );
     wxASSERT( !m_workFile );
 
     m_pageNumbers.push_back( aPageNumber );
-    m_pageName = aPageName;
+    m_pageName = aPageName.IsEmpty()
+                    ? wxString::Format( _( "Page %s" ), aPageNumber )
+                    : wxString::Format( _( "%s (Page %s)" ), aPageName, aPageNumber );
+    m_parentPageName = aParentPageName.IsEmpty()
+                    ? wxString::Format( _( "Page %s" ), aParentPageNumber )
+                    : wxString::Format( _( "%s (Page %s)" ), aParentPageName, aParentPageNumber );
 
     // Compute the paper size in IUs
     m_paperSize = m_pageInfo.GetSizeMils();
     m_paperSize.x *= 10.0 / m_iuPerDeviceUnit;
     m_paperSize.y *= 10.0 / m_iuPerDeviceUnit;
 
+    // Set m_currentPenWidth to a unused value to ensure the pen width
+    // will be initialized to a the right value in pdf file by the first item to plot
+    m_currentPenWidth = 0;
+
     // Open the content stream; the page object will go later
     m_pageStreamHandle = startPdfStream();
 
@@ -774,15 +788,19 @@ void PDF_PLOTTER::ClosePage()
     auto iuToPdfUserSpace =
             [&]( const VECTOR2I& aCoord ) -> VECTOR2D
             {
-                VECTOR2D retval = VECTOR2D( aCoord ) * PTsPERMIL / ( m_IUsPerDecimil * 10 );
+                VECTOR2D pos = VECTOR2D( aCoord ) * PTsPERMIL / ( m_IUsPerDecimil * 10 );
+
                 // PDF y=0 is at bottom of page, invert coordinate
-                retval.y = psPaperSize.y - retval.y;
+                VECTOR2D retval( pos.x, psPaperSize.y - pos.y );
 
                 // The pdf plot can be mirrored (from left to right). So mirror the
                 // x coordinate if m_plotMirror is set
                 if( m_plotMirror )
                 {
-                    retval.x = ( psPaperSize.x - retval.x );
+                    if( m_mirrorIsHorizontal )
+                        retval.x = ( psPaperSize.x - pos.x );
+                    else
+                        retval.y = pos.y;
                 }
 
                 return retval;
@@ -882,21 +900,34 @@ void PDF_PLOTTER::ClosePage()
     // Mark the page stream as idle
     m_pageStreamHandle = 0;
 
-    wxString pageOutlineName = wxEmptyString;
+    int                        actionHandle = emitGoToAction( pageHandle );
+    PDF_PLOTTER::OUTLINE_NODE* parent_node = m_outlineRoot.get();
 
-    if( m_pageName.IsEmpty() )
+    if( !m_parentPageName.IsEmpty() )
     {
-        pageOutlineName = wxString::Format( _( "Page %s" ), m_pageNumbers.back() );
-    }
-    else
-    {
-        pageOutlineName = wxString::Format( _( "%s (Page %s)" ), m_pageName, m_pageNumbers.back() );
+        // Search for the parent node iteratively through the entire tree
+        std::stack<OUTLINE_NODE*> nodes;
+        nodes.push( m_outlineRoot.get() );
+
+        while( !nodes.empty() )
+        {
+            OUTLINE_NODE* node = nodes.top();
+            nodes.pop();
+
+            // Check if this node matches
+            if( node->title == m_parentPageName )
+            {
+                parent_node = node;
+                break;
+            }
+
+            // Add all children to the stack
+            for( OUTLINE_NODE* child : node->children )
+                nodes.push( child );
+        }
     }
 
-
-    int           actionHandle = emitGoToAction( pageHandle );
-    OUTLINE_NODE* pageOutlineNode =
-            addOutlineNode( m_outlineRoot.get(), actionHandle, pageOutlineName );
+    OUTLINE_NODE* pageOutlineNode = addOutlineNode( parent_node, actionHandle, m_pageName );
 
     // let's reorg the symbol bookmarks under a page handle
     // let's reorg the symbol bookmarks under a page handle
@@ -1117,7 +1148,9 @@ int PDF_PLOTTER::emitOutline()
 
 bool PDF_PLOTTER::EndPlot()
 {
-    wxASSERT( m_outputFile );
+    // We can end up here if there was nothing to plot
+    if( !m_outputFile )
+        return false;
 
     // Close the current page (often the only one)
     ClosePage();
@@ -1275,8 +1308,8 @@ bool PDF_PLOTTER::EndPlot()
 
     for( const auto& [ linkHandle, linkPair ] : m_hyperlinkHandles )
     {
-        const BOX2D&    box = linkPair.first;
-        const wxString& url = linkPair.second;
+        BOX2D    box = linkPair.first;
+        wxString url = linkPair.second;
 
         startPdfObject( linkHandle );
 
@@ -1316,6 +1349,9 @@ bool PDF_PLOTTER::EndPlot()
         }
         else
         {
+            if( m_project )
+                url = ResolveUriByEnvVars( url, m_project );
+
             fprintf( m_outputFile,
                      "/A << /Type /Action /S /URI /URI %s >>\n"
                      ">>\n",
@@ -1482,7 +1518,6 @@ function ShM(aEntries) {
             ">>\n", (long) m_pageHandles.size() );
     closePdfObject();
 
-
     // The info dictionary
     int infoDictHandle = startPdfObject();
     char date_buf[250];
@@ -1501,10 +1536,14 @@ function ShM(aEntries) {
              "/Producer (KiCad PDF)\n"
              "/CreationDate (%s)\n"
              "/Creator %s\n"
-             "/Title %s\n",
+             "/Title %s\n"
+             "/Author %s\n"
+             "/Subject %s\n",
              date_buf,
              encodeStringForPlotter( m_creator ).c_str(),
-             encodeStringForPlotter( m_title ).c_str() );
+             encodeStringForPlotter( m_title ).c_str(),
+             encodeStringForPlotter( m_author ).c_str(),
+             encodeStringForPlotter( m_subject ).c_str() );
 
     fputs( ">>\n", m_outputFile );
     closePdfObject();
