@@ -41,9 +41,7 @@ class PROPERTIES_PANEL : public wxPanel
 public:
     PROPERTIES_PANEL( wxWindow* aParent, EDA_BASE_FRAME* aFrame );
 
-    virtual ~PROPERTIES_PANEL()
-    {
-    }
+    virtual ~PROPERTIES_PANEL();
 
     virtual void UpdateData() = 0;
 
@@ -69,8 +67,6 @@ public:
     void SetSplitterProportion( float aProportion );
     float SplitterProportion() const { return m_splitter_key_proportion; }
 
-    void OnLanguageChanged();
-
 protected:
     /**
      * Generates the property grid for a given selection of items.
@@ -82,11 +78,13 @@ protected:
     virtual wxPGProperty* createPGProperty( const PROPERTY_BASE* aProperty ) const = 0;
 
     // Event handlers
-    virtual void valueChanging( wxPropertyGridEvent& aEvent ) {}
-    virtual void valueChanged( wxPropertyGridEvent& aEvent ) {}
+    virtual void valueChanging( wxPropertyGridEvent& aEvent ) { aEvent.Skip(); }
+    virtual void valueChanged( wxPropertyGridEvent& aEvent ) { aEvent.Skip(); }
     void onCharHook( wxKeyEvent& aEvent );
     void onShow( wxShowEvent& aEvent );
 
+    virtual void OnLanguageChanged( wxCommandEvent& aEvent );
+
     /**
      * Utility to fetch a property value and convert to wxVariant
      * Precondition: aItem is known to have property aProperty
@@ -106,11 +104,14 @@ protected:
     bool extractValueAndWritability( const SELECTION& aSelection, PROPERTY_BASE* aProperty,
                                      wxVariant& aValue, bool& aWritable );
 
+public:
+    int                         m_SuppressGridChangeEvents;
+
 protected:
     std::vector<PROPERTY_BASE*> m_displayed;
-    wxPropertyGrid* m_grid;
-    EDA_BASE_FRAME* m_frame;
-    wxStaticText* m_caption;
+    wxPropertyGrid*             m_grid;
+    EDA_BASE_FRAME*             m_frame;
+    wxStaticText*               m_caption;
 
     /// Proportion of the grid column splitter that is used for the key column (0.0 - 1.0)
     float m_splitter_key_proportion;
