@@ -1,7 +1,7 @@
 /*
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
- * Copyright (C) 2004-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -32,25 +32,22 @@
 #include <kiface_base.h>
 #include <pin_numbers.h>
 #include <string_utils.h>
-#include <menus_helpers.h>
 #include <kiplatform/ui.h>
 #include <widgets/grid_icon_text_helpers.h>
 #include <widgets/grid_combobox.h>
 #include <widgets/std_bitmap_button.h>
 #include <settings/settings_manager.h>
-#include <ee_collectors.h>
+#include <sch_collectors.h>
 #include <symbol_library.h>
 #include <fields_grid_table.h>
 #include <sch_edit_frame.h>
 #include <sch_reference_list.h>
 #include <schematic.h>
+#include <sch_commit.h>
 #include <tool/tool_manager.h>
 #include <tool/actions.h>
-#include <math/vector2d.h>
 
-#ifdef KICAD_SPICE
 #include <dialog_sim_model.h>
-#endif /* KICAD_SPICE */
 
 
 wxDEFINE_EVENT( SYMBOL_DELAY_FOCUS, wxCommandEvent );
@@ -68,14 +65,8 @@ enum PIN_TABLE_COL_ORDER
 };
 
 
-class SCH_PIN_TABLE_DATA_MODEL : public wxGridTableBase, public std::vector<SCH_PIN>
+class SCH_PIN_TABLE_DATA_MODEL : public WX_GRID_TABLE_BASE, public std::vector<SCH_PIN>
 {
-protected:
-    std::vector<wxGridCellAttr*> m_nameAttrs;
-    wxGridCellAttr*              m_readOnlyAttr;
-    wxGridCellAttr*              m_typeAttr;
-    wxGridCellAttr*              m_shapeAttr;
-
 public:
     SCH_PIN_TABLE_DATA_MODEL() :
             m_readOnlyAttr( nullptr ),
@@ -109,20 +100,21 @@ public:
 
         for( const SCH_PIN& pin : *this )
         {
-            LIB_PIN*        lib_pin = pin.GetLibPin();
+            SCH_PIN*        lib_pin = pin.GetLibPin();
             wxGridCellAttr* attr = nullptr;
 
             if( lib_pin->GetAlternates().empty() )
             {
                 attr = new wxGridCellAttr;
                 attr->SetReadOnly( true );
+                attr->SetBackgroundColour( KIPLATFORM::UI::GetDialogBGColour() );
             }
             else
             {
                 wxArrayString choices;
                 choices.push_back( lib_pin->GetName() );
 
-                for( const std::pair<const wxString, LIB_PIN::ALT>& alt : lib_pin->GetAlternates() )
+                for( const std::pair<const wxString, SCH_PIN::ALT>& alt : lib_pin->GetAlternates() )
                     choices.push_back( alt.first );
 
                 attr = new wxGridCellAttr();
@@ -170,6 +162,12 @@ public:
         return false;   // don't allow adjacent cell overflow, even if we are actually empty
     }
 
+    bool CanSetValueAs( int aRow, int aCol, const wxString& aTypeName ) override
+    {
+        // Don't accept random values; must use the popup to change to a known alternate
+        return false;
+    }
+
     wxString GetValue( int aRow, int aCol ) override
     {
         return GetValue( at( aRow ), aCol );
@@ -177,37 +175,46 @@ public:
 
     static wxString GetValue( const SCH_PIN& aPin, int aCol )
     {
+        if( aCol == COL_ALT_NAME )
+        {
+            if( aPin.GetLibPin()->GetAlternates().empty() )
+                return wxEmptyString;
+            else if( aPin.GetAlt().IsEmpty() )
+                return aPin.GetName();
+            else
+                return aPin.GetAlt();
+        }
+
         switch( aCol )
         {
         case COL_NUMBER:    return aPin.GetNumber();
         case COL_BASE_NAME: return aPin.GetLibPin()->GetName();
-        case COL_ALT_NAME:  return aPin.GetAlt();
         case COL_TYPE:      return PinTypeNames()[static_cast<int>( aPin.GetType() )];
         case COL_SHAPE:     return PinShapeNames()[static_cast<int>( aPin.GetShape() )];
         default:   wxFAIL;  return wxEmptyString;
         }
     }
 
-    wxGridCellAttr* GetAttr( int aRow, int aCol, wxGridCellAttr::wxAttrKind  ) override
+    wxGridCellAttr* GetAttr( int aRow, int aCol, wxGridCellAttr::wxAttrKind aKind ) override
     {
         switch( aCol )
         {
         case COL_NUMBER:
         case COL_BASE_NAME:
             m_readOnlyAttr->IncRef();
-            return m_readOnlyAttr;
+            return enhanceAttr( m_readOnlyAttr, aRow, aCol, aKind );
 
         case COL_ALT_NAME:
             m_nameAttrs[ aRow ]->IncRef();
-            return m_nameAttrs[ aRow ];
+            return enhanceAttr( m_nameAttrs[ aRow ], aRow, aCol, aKind );
 
         case COL_TYPE:
             m_typeAttr->IncRef();
-            return m_typeAttr;
+            return enhanceAttr( m_typeAttr, aRow, aCol, aKind );
 
         case COL_SHAPE:
             m_shapeAttr->IncRef();
-            return m_shapeAttr;
+            return enhanceAttr( m_shapeAttr, aRow, aCol, aKind );
 
         default:
             wxFAIL;
@@ -291,6 +298,12 @@ public:
                        return compare( lhs, rhs, aSortCol, ascending );
                    } );
     }
+
+protected:
+    std::vector<wxGridCellAttr*> m_nameAttrs;
+    wxGridCellAttr*              m_readOnlyAttr;
+    wxGridCellAttr*              m_typeAttr;
+    wxGridCellAttr*              m_shapeAttr;
 };
 
 
@@ -300,7 +313,8 @@ DIALOG_SYMBOL_PROPERTIES::DIALOG_SYMBOL_PROPERTIES( SCH_EDIT_FRAME* aParent,
         m_symbol( nullptr ),
         m_part( nullptr ),
         m_fieldsSize( 0, 0 ),
-        m_lastRequestedSize( 0, 0 ),
+        m_lastRequestedFieldsSize( 0, 0 ),
+        m_lastRequestedPinsSize( 0, 0 ),
         m_editorShown( false ),
         m_fields( nullptr ),
         m_dataModel( nullptr )
@@ -313,12 +327,7 @@ DIALOG_SYMBOL_PROPERTIES::DIALOG_SYMBOL_PROPERTIES( SCH_EDIT_FRAME* aParent,
     // so we need to handle m_part == nullptr
     // wxASSERT( m_part );
 
-    m_fields = new FIELDS_GRID_TABLE<SCH_FIELD>( this, aParent, m_fieldsGrid, m_symbol );
-
-#ifndef KICAD_SPICE
-    m_cbExcludeFromSim->Hide();
-    m_spiceFieldsButton->Hide();
-#endif /* not KICAD_SPICE */
+    m_fields = new FIELDS_GRID_TABLE( this, aParent, m_fieldsGrid, m_symbol );
 
     // Give a bit more room for combobox editors
     m_fieldsGrid->SetDefaultRowSize( m_fieldsGrid->GetDefaultRowSize() + 4 );
@@ -326,6 +335,7 @@ DIALOG_SYMBOL_PROPERTIES::DIALOG_SYMBOL_PROPERTIES( SCH_EDIT_FRAME* aParent,
 
     m_fieldsGrid->SetTable( m_fields );
     m_fieldsGrid->PushEventHandler( new FIELDS_GRID_TRICKS( m_fieldsGrid, this,
+                                                            { &aParent->Schematic(), m_part },
                                                             [&]( wxCommandEvent& aEvent )
                                                             {
                                                                 OnAddField( aEvent );
@@ -333,15 +343,13 @@ DIALOG_SYMBOL_PROPERTIES::DIALOG_SYMBOL_PROPERTIES( SCH_EDIT_FRAME* aParent,
     m_fieldsGrid->SetSelectionMode( wxGrid::wxGridSelectRows );
 
     // Show/hide columns according to user's preference
-    EESCHEMA_SETTINGS* cfg = dynamic_cast<EESCHEMA_SETTINGS*>( Kiface().KifaceSettings() );
-
-    if( cfg )
+    if( EESCHEMA_SETTINGS* cfg = dynamic_cast<EESCHEMA_SETTINGS*>( Kiface().KifaceSettings() ) )
     {
-        m_shownColumns = cfg->m_Appearance.edit_symbol_visible_columns;
-        m_fieldsGrid->ShowHideColumns( m_shownColumns );
+        m_fieldsGrid->ShowHideColumns( cfg->m_Appearance.edit_symbol_visible_columns );
+        m_shownColumns = m_fieldsGrid->GetShownColumns();
     }
 
-    if( m_part && m_part->HasConversion() )
+    if( m_part && m_part->HasAlternateBodyStyle() )
     {
         // DeMorgan conversions are a subclass of alternate pin assignments, so don't allow
         // free-form alternate assignments as well.  (We won't know how to map the alternates
@@ -364,17 +372,22 @@ DIALOG_SYMBOL_PROPERTIES::DIALOG_SYMBOL_PROPERTIES( SCH_EDIT_FRAME* aParent,
         m_pinGrid->SetTable( m_dataModel );
     }
 
+    if( m_part && m_part->IsPower() )
+        m_spiceFieldsButton->Hide();
+
     m_pinGrid->PushEventHandler( new GRID_TRICKS( m_pinGrid ) );
     m_pinGrid->SetSelectionMode( wxGrid::wxGridSelectRows );
 
+    m_tcLibraryID->SetBackgroundColour( KIPLATFORM::UI::GetDialogBGColour() );
+
     wxToolTip::Enable( true );
     SetupStandardButtons();
 
     // Configure button logos
-    m_bpAdd->SetBitmap( KiBitmap( BITMAPS::small_plus ) );
-    m_bpDelete->SetBitmap( KiBitmap( BITMAPS::small_trash ) );
-    m_bpMoveUp->SetBitmap( KiBitmap( BITMAPS::small_up ) );
-    m_bpMoveDown->SetBitmap( KiBitmap( BITMAPS::small_down ) );
+    m_bpAdd->SetBitmap( KiBitmapBundle( BITMAPS::small_plus ) );
+    m_bpDelete->SetBitmap( KiBitmapBundle( BITMAPS::small_trash ) );
+    m_bpMoveUp->SetBitmap( KiBitmapBundle( BITMAPS::small_up ) );
+    m_bpMoveDown->SetBitmap( KiBitmapBundle( BITMAPS::small_down ) );
 
     // wxFormBuilder doesn't include this event...
     m_fieldsGrid->Bind( wxEVT_GRID_CELL_CHANGING, &DIALOG_SYMBOL_PROPERTIES::OnGridCellChanging,
@@ -394,11 +407,9 @@ DIALOG_SYMBOL_PROPERTIES::DIALOG_SYMBOL_PROPERTIES( SCH_EDIT_FRAME* aParent,
 
 DIALOG_SYMBOL_PROPERTIES::~DIALOG_SYMBOL_PROPERTIES()
 {
-    EESCHEMA_SETTINGS* cfg = dynamic_cast<EESCHEMA_SETTINGS*>( Kiface().KifaceSettings() );
-
-    if( cfg )
+    if( EESCHEMA_SETTINGS* cfg = dynamic_cast<EESCHEMA_SETTINGS*>( Kiface().KifaceSettings() ) )
     {
-        cfg->m_Appearance.edit_symbol_visible_columns = m_fieldsGrid->GetShownColumns();
+        cfg->m_Appearance.edit_symbol_visible_columns = m_fieldsGrid->GetShownColumnsAsString();
         cfg->m_Appearance.edit_symbol_width = GetSize().x;
         cfg->m_Appearance.edit_symbol_height = GetSize().y;
     }
@@ -442,6 +453,8 @@ bool DIALOG_SYMBOL_PROPERTIES::TransferDataToWindow()
         // change offset to be symbol-relative
         field.Offset( -m_symbol->GetPosition() );
 
+        field.SetText( m_symbol->Schematic()->ConvertKIIDsToRefs( field.GetText() ) );
+
         defined.insert( field.GetName() );
         m_fields->push_back( field );
     }
@@ -452,14 +465,14 @@ bool DIALOG_SYMBOL_PROPERTIES::TransferDataToWindow()
     {
         if( defined.count( templateFieldname.m_Name ) <= 0 )
         {
-            SCH_FIELD field( wxPoint( 0, 0 ), -1, m_symbol, templateFieldname.m_Name );
+            SCH_FIELD field( VECTOR2I( 0, 0 ), -1, m_symbol, templateFieldname.m_Name );
             field.SetVisible( templateFieldname.m_Visible );
             m_fields->push_back( field );
         }
     }
 
     // notify the grid
-    wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, m_fields->size() );
+    wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, m_fields->GetNumberRows() );
     m_fieldsGrid->ProcessTableMessage( msg );
     AdjustFieldsGridColumns();
 
@@ -468,14 +481,14 @@ bool DIALOG_SYMBOL_PROPERTIES::TransferDataToWindow()
     {
         // Ensure symbol unit is the currently selected unit (mandatory in complex hierarchies)
         // from the current sheet path, because it can be modified by previous calculations
-        m_symbol->UpdateUnit( m_symbol->GetUnitSelection( &GetParent()->GetCurrentSheet() ) );
+        m_symbol->SetUnit( m_symbol->GetUnitSelection( &GetParent()->GetCurrentSheet() ) );
 
         for( int ii = 1; ii <= m_symbol->GetUnitCount(); ii++ )
         {
             if( m_symbol->HasUnitDisplayName( ii ) )
                 m_unitChoice->Append( m_symbol->GetUnitDisplayName( ii ) );
             else
-                m_unitChoice->Append( LIB_SYMBOL::SubReference( ii, false ) );
+                m_unitChoice->Append( m_symbol->SubReference( ii, false ) );
         }
 
         if( m_symbol->GetUnit() <= ( int )m_unitChoice->GetCount() )
@@ -487,9 +500,9 @@ bool DIALOG_SYMBOL_PROPERTIES::TransferDataToWindow()
         m_unitChoice->Enable( false );
     }
 
-    if( m_part && m_part->HasConversion() )
+    if( m_part && m_part->HasAlternateBodyStyle() )
     {
-        if( m_symbol->GetConvert() > LIB_ITEM::LIB_CONVERT::BASE )
+        if( m_symbol->GetBodyStyle() > BODY_STYLE::BASE )
             m_cbAlternateSymbol->SetValue( true );
     }
     else
@@ -518,17 +531,15 @@ bool DIALOG_SYMBOL_PROPERTIES::TransferDataToWindow()
     case SYM_MIRROR_Y: m_mirrorCtrl->SetSelection( 2 ); break;
     }
 
-#ifdef KICAD_SPICE
-    m_cbExcludeFromSim->SetValue( m_symbol->GetFieldText( SIM_ENABLE_FIELD ) == wxS( "0" ) );
-#endif
-    m_cbExcludeFromBom->SetValue( !m_symbol->GetIncludeInBom() );
-    m_cbExcludeFromBoard->SetValue( !m_symbol->GetIncludeOnBoard() );
+    m_cbExcludeFromSim->SetValue( m_symbol->GetExcludedFromSim() );
+    m_cbExcludeFromBom->SetValue( m_symbol->GetExcludedFromBOM() );
+    m_cbExcludeFromBoard->SetValue( m_symbol->GetExcludedFromBoard() );
     m_cbDNP->SetValue( m_symbol->GetDNP() );
 
     if( m_part )
     {
-        m_ShowPinNumButt->SetValue( m_part->ShowPinNumbers() );
-        m_ShowPinNameButt->SetValue( m_part->ShowPinNames() );
+        m_ShowPinNumButt->SetValue( m_part->GetShowPinNumbers() );
+        m_ShowPinNameButt->SetValue( m_part->GetShowPinNames() );
     }
 
     // Set the symbol's library name.
@@ -542,50 +553,8 @@ bool DIALOG_SYMBOL_PROPERTIES::TransferDataToWindow()
 }
 
 
-void DIALOG_SYMBOL_PROPERTIES::OnExcludeFromSimulation( wxCommandEvent& event )
-{
-#ifdef KICAD_SPICE
-    int simEnableFieldRow = -1;
-
-    for( int ii = MANDATORY_FIELDS; ii < m_fieldsGrid->GetNumberRows(); ++ii )
-    {
-        if( m_fieldsGrid->GetCellValue( ii, FDC_NAME ) == SIM_ENABLE_FIELD )
-            simEnableFieldRow = ii;
-    }
-
-    if( event.IsChecked() )
-    {
-        if( simEnableFieldRow == -1 )
-        {
-            simEnableFieldRow = (int) m_fields->size();
-            m_fields->emplace_back( VECTOR2I( 0, 0 ), simEnableFieldRow, m_symbol, SIM_ENABLE_FIELD );
-
-            // notify the grid
-            wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_APPENDED, 1 );
-            m_fieldsGrid->ProcessTableMessage( msg );
-        }
-
-        m_fieldsGrid->SetCellValue( simEnableFieldRow, FDC_VALUE, wxT( "0" ) );
-        m_fieldsGrid->SetCellValue( simEnableFieldRow, FDC_SHOWN, wxT( "0" ) );
-        m_fieldsGrid->SetCellValue( simEnableFieldRow, FDC_SHOW_NAME, wxT( "0" ) );
-    }
-    else if( simEnableFieldRow >= 0 )
-    {
-        m_fields->erase( m_fields->begin() + simEnableFieldRow );
-
-        // notify the grid
-        wxGridTableMessage msg( m_fields, wxGRIDTABLE_NOTIFY_ROWS_DELETED, simEnableFieldRow, 1 );
-        m_fieldsGrid->ProcessTableMessage( msg );
-    }
-
-    OnModify();
-#endif
-}
-
-
 void DIALOG_SYMBOL_PROPERTIES::OnEditSpiceModel( wxCommandEvent& event )
 {
-#ifdef KICAD_SPICE
     if( !m_fieldsGrid->CommitPendingChanges() )
         return;
 
@@ -594,7 +563,7 @@ void DIALOG_SYMBOL_PROPERTIES::OnEditSpiceModel( wxCommandEvent& event )
     for( const SCH_FIELD& field : *m_fields )
         fields.emplace_back( field );
 
-    DIALOG_SIM_MODEL dialog( this, *m_symbol, fields );
+    DIALOG_SIM_MODEL dialog( this, m_parentFrame, *m_symbol, fields );
 
     if( dialog.ShowModal() != wxID_OK )
         return;
@@ -648,7 +617,6 @@ void DIALOG_SYMBOL_PROPERTIES::OnEditSpiceModel( wxCommandEvent& event )
 
     OnModify();
     m_fieldsGrid->ForceRefresh();
-#endif /* KICAD_SPICE */
 }
 
 
@@ -667,22 +635,15 @@ bool DIALOG_SYMBOL_PROPERTIES::Validate()
     if( !m_fieldsGrid->CommitPendingChanges() || !m_fieldsGrid->Validate() )
         return false;
 
-    if( !SCH_SYMBOL::IsReferenceStringValid( m_fields->at( REFERENCE_FIELD ).GetText() ) )
-    {
-        DisplayErrorMessage( this, _( "References must start with a letter." ) );
-
-        wxCommandEvent *evt = new wxCommandEvent( SYMBOL_DELAY_FOCUS );
-        evt->SetClientData( new VECTOR2I( REFERENCE_FIELD, FDC_VALUE ) );
-        QueueEvent( evt );
-
-        return false;
-    }
-
     // Check for missing field names.
-    for( size_t i = MANDATORY_FIELDS;  i < m_fields->size(); ++i )
+    for( size_t i = 0; i < m_fields->size(); ++i )
     {
         SCH_FIELD& field = m_fields->at( i );
-        wxString   fieldName = field.GetName( false );
+
+        if( field.IsMandatory() )
+            continue;
+
+        wxString fieldName = field.GetName( false );
 
         if( fieldName.IsEmpty() )
         {
@@ -711,25 +672,27 @@ bool DIALOG_SYMBOL_PROPERTIES::TransferDataFromWindow()
     if( !m_pinGrid->CommitPendingChanges() )
         return false;
 
+    SCH_COMMIT  commit( GetParent() );
     SCH_SCREEN* currentScreen = GetParent()->GetScreen();
+    bool        replaceOnCurrentScreen;
     wxCHECK( currentScreen, false );
 
     // This needs to be done before the LIB_ID is changed to prevent stale library symbols in
     // the schematic file.
-    currentScreen->Remove( m_symbol );
+    replaceOnCurrentScreen = currentScreen->Remove( m_symbol );
 
     // save old cmp in undo list if not already in edit, or moving ...
     if( m_symbol->GetEditFlags() == 0 )
-        GetParent()->SaveCopyInUndoList( currentScreen, m_symbol, UNDO_REDO::CHANGED, false );
+        commit.Modify( m_symbol, currentScreen );
 
     // Save current flags which could be modified by next change settings
     EDA_ITEM_FLAGS flags = m_symbol->GetFlags();
 
     // For symbols with multiple shapes (De Morgan representation) Set the selected shape:
     if( m_cbAlternateSymbol->IsEnabled() && m_cbAlternateSymbol->GetValue() )
-        m_symbol->SetConvert( LIB_ITEM::LIB_CONVERT::DEMORGAN );
+        m_symbol->SetBodyStyle( BODY_STYLE::DEMORGAN );
     else
-        m_symbol->SetConvert( LIB_ITEM::LIB_CONVERT::BASE );
+        m_symbol->SetBodyStyle( BODY_STYLE::BASE );
 
     //Set the part selection in multiple part per package
     int unit_selection = m_unitChoice->IsEnabled() ? m_unitChoice->GetSelection() + 1 : 1;
@@ -751,39 +714,48 @@ bool DIALOG_SYMBOL_PROPERTIES::TransferDataFromWindow()
     case 2: m_symbol->SetOrientation( SYM_MIRROR_Y ); break;
     }
 
-    if( m_part )
-    {
-        m_part->SetShowPinNames( m_ShowPinNameButt->GetValue() );
-        m_part->SetShowPinNumbers( m_ShowPinNumButt->GetValue() );
-    }
+    m_symbol->SetShowPinNames( m_ShowPinNameButt->GetValue() );
+    m_symbol->SetShowPinNumbers( m_ShowPinNumButt->GetValue() );
 
     // Restore m_Flag modified by SetUnit() and other change settings from the dialog
     m_symbol->ClearFlags();
     m_symbol->SetFlags( flags );
 
     // change all field positions from relative to absolute
-    for( unsigned i = 0;  i < m_fields->size();  ++i )
-        m_fields->at( i ).Offset( m_symbol->GetPosition() );
+    for( SCH_FIELD& field : *m_fields )
+    {
+        field.Offset( m_symbol->GetPosition() );
+        field.SetText( m_symbol->Schematic()->ConvertRefsToKIIDs( field.GetText() ) );
+    }
 
     SCH_FIELDS& fields = m_symbol->GetFields();
-
     fields.clear();
 
-    for( size_t i = 0; i < m_fields->size(); ++i )
-        fields.push_back( m_fields->at( i ) );
+    for( SCH_FIELD& field : *m_fields )
+    {
+        const wxString& fieldName = field.GetCanonicalName();
+
+        if( fieldName.IsEmpty() && field.GetText().IsEmpty() )
+            continue;
+        else if( fieldName.IsEmpty() )
+            field.SetName( _( "untitled" ) );
+
+        fields.push_back( field );
+    }
 
     // Reference has a specific initialization, depending on the current active sheet
     // because for a given symbol, in a complex hierarchy, there are more than one
     // reference.
     m_symbol->SetRef( &GetParent()->GetCurrentSheet(), m_fields->at( REFERENCE_FIELD ).GetText() );
 
-    // Similar for Value and Footprint, except that the GUI behaviour is that they are kept
+    // Similar for Value and Footprint, except that the GUI behavior is that they are kept
     // in sync between multiple instances.
     m_symbol->SetValueFieldText( m_fields->at( VALUE_FIELD ).GetText() );
     m_symbol->SetFootprintFieldText(  m_fields->at( FOOTPRINT_FIELD ).GetText() );
 
-    m_symbol->SetIncludeInBom( !m_cbExcludeFromBom->IsChecked() );
-    m_symbol->SetIncludeOnBoard( !m_cbExcludeFromBoard->IsChecked() );
+    m_symbol->SetExcludedFromSim( m_cbExcludeFromSim->IsChecked() );
+    m_symbol->SetExcludedFromBOM( m_cbExcludeFromBom->IsChecked() );
+    m_symbol->SetExcludedFromBoard( m_cbExcludeFromBoard->IsChecked() );
     m_symbol->SetDNP( m_cbDNP->IsChecked() );
 
     // Update any assignments
@@ -801,83 +773,13 @@ bool DIALOG_SYMBOL_PROPERTIES::TransferDataFromWindow()
 
     // Keep fields other than the reference, include/exclude flags, and alternate pin assignements
     // in sync in multi-unit parts.
-    if( m_symbol->GetUnitCount() > 1 && m_symbol->IsAnnotated( &GetParent()->GetCurrentSheet() ) )
-    {
-        wxString ref = m_symbol->GetRef( &GetParent()->GetCurrentSheet() );
-        int      unit = m_symbol->GetUnit();
-        LIB_ID   libId = m_symbol->GetLibId();
+    m_symbol->SyncOtherUnits( GetParent()->GetCurrentSheet(), commit, nullptr );
 
-        for( SCH_SHEET_PATH& sheet : GetParent()->Schematic().GetSheets() )
-        {
-            SCH_SCREEN*              screen = sheet.LastScreen();
-            std::vector<SCH_SYMBOL*> otherUnits;
-            constexpr bool           appendUndo = true;
+    if( replaceOnCurrentScreen )
+        currentScreen->Append( m_symbol );
 
-            CollectOtherUnits( ref, unit, libId, sheet, &otherUnits );
-
-            for( SCH_SYMBOL* otherUnit : otherUnits )
-            {
-                GetParent()->SaveCopyInUndoList( screen, otherUnit, UNDO_REDO::CHANGED,
-                                                 appendUndo );
-                otherUnit->SetValueFieldText( m_fields->at( VALUE_FIELD ).GetText() );
-                otherUnit->SetFootprintFieldText( m_fields->at( FOOTPRINT_FIELD ).GetText() );
-
-                for( size_t ii = DATASHEET_FIELD; ii < m_fields->size(); ++ii )
-                {
-                    SCH_FIELD* otherField = otherUnit->FindField( m_fields->at( ii ).GetName() );
-
-                    if( otherField )
-                    {
-                        otherField->SetText( m_fields->at( ii ).GetText() );
-                    }
-                    else
-                    {
-                        SCH_FIELD newField( m_fields->at( ii ) );
-                        const_cast<KIID&>( newField.m_Uuid ) = KIID();
-
-                        newField.Offset( -m_symbol->GetPosition() );
-                        newField.Offset( otherUnit->GetPosition() );
-
-                        newField.SetParent( otherUnit );
-                        otherUnit->AddField( newField );
-                    }
-                }
-
-                for( size_t ii = otherUnit->GetFields().size() - 1; ii > DATASHEET_FIELD; ii-- )
-                {
-                    SCH_FIELD& otherField = otherUnit->GetFields().at( ii );
-
-                    if( !m_symbol->FindField( otherField.GetName() ) )
-                        otherUnit->GetFields().erase( otherUnit->GetFields().begin() + ii );
-                }
-
-                otherUnit->SetIncludeInBom( !m_cbExcludeFromBom->IsChecked() );
-                otherUnit->SetIncludeOnBoard( !m_cbExcludeFromBoard->IsChecked() );
-                otherUnit->SetDNP( m_cbDNP->IsChecked() );
-
-                if( m_dataModel )
-                {
-                    for( const SCH_PIN& model_pin : *m_dataModel )
-                    {
-                        SCH_PIN* src_pin = otherUnit->GetPin( model_pin.GetNumber() );
-
-                        if( src_pin )
-                            src_pin->SetAlt( model_pin.GetAlt() );
-                    }
-                }
-
-                GetParent()->UpdateItem( otherUnit, false, true );
-            }
-        }
-    }
-
-    currentScreen->Append( m_symbol );
-    GetParent()->TestDanglingEnds();
-    GetParent()->UpdateItem( m_symbol, false, true );
-    GetParent()->OnModify();
-
-    // This must go after OnModify() so that the connectivity graph will have been updated.
-    GetParent()->GetToolManager()->PostEvent( EVENTS::SelectedItemsModified );
+    if( !commit.Empty() )
+        commit.Push( _( "Edit Symbol Properties" ) );
 
     return true;
 }
@@ -942,11 +844,11 @@ void DIALOG_SYMBOL_PROPERTIES::OnAddField( wxCommandEvent& event )
 
     SCHEMATIC_SETTINGS& settings = m_symbol->Schematic()->Settings();
     int                 fieldID = (int) m_fields->size();
-    SCH_FIELD           newField( wxPoint( 0, 0 ), fieldID, m_symbol,
-                                  TEMPLATE_FIELDNAME::GetDefaultFieldName( fieldID, DO_TRANSLATE ) );
+    SCH_FIELD           newField( VECTOR2I(), fieldID, m_symbol, GetUserFieldName( fieldID, DO_TRANSLATE ) );
 
     newField.SetTextAngle( m_fields->at( REFERENCE_FIELD ).GetTextAngle() );
-    newField.SetTextSize( wxSize( settings.m_DefaultTextSize, settings.m_DefaultTextSize ) );
+    newField.SetTextSize( VECTOR2I( settings.m_DefaultTextSize, settings.m_DefaultTextSize ) );
+    newField.SetVisible( false );
 
     m_fields->push_back( newField );
 
@@ -976,10 +878,10 @@ void DIALOG_SYMBOL_PROPERTIES::OnDeleteField( wxCommandEvent& event )
 
     for( int row : selectedRows )
     {
-        if( row < MANDATORY_FIELDS )
+        if( row < m_fields->GetMandatoryRowCount() )
         {
             DisplayError( this, wxString::Format( _( "The first %d fields are mandatory." ),
-                                                  MANDATORY_FIELDS ) );
+                                                  m_fields->GetMandatoryRowCount() ) );
             return;
         }
     }
@@ -1016,7 +918,7 @@ void DIALOG_SYMBOL_PROPERTIES::OnMoveUp( wxCommandEvent& event )
 
     int i = m_fieldsGrid->GetGridCursorRow();
 
-    if( i > MANDATORY_FIELDS )
+    if( i > m_fields->GetMandatoryRowCount() )
     {
         SCH_FIELD tmp = m_fields->at( (unsigned) i );
         m_fields->erase( m_fields->begin() + i, m_fields->begin() + i + 1 );
@@ -1043,7 +945,7 @@ void DIALOG_SYMBOL_PROPERTIES::OnMoveDown( wxCommandEvent& event )
 
     int i = m_fieldsGrid->GetGridCursorRow();
 
-    if( i >= MANDATORY_FIELDS && i < m_fieldsGrid->GetNumberRows() - 1 )
+    if( i >= m_fields->GetMandatoryRowCount() && i < m_fieldsGrid->GetNumberRows() - 1 )
     {
         SCH_FIELD tmp = m_fields->at( (unsigned) i );
         m_fields->erase( m_fields->begin() + i, m_fields->begin() + i + 1 );
@@ -1168,7 +1070,7 @@ void DIALOG_SYMBOL_PROPERTIES::AdjustPinsGridColumns()
 
 void DIALOG_SYMBOL_PROPERTIES::OnUpdateUI( wxUpdateUIEvent& event )
 {
-    wxString shownColumns = m_fieldsGrid->GetShownColumns();
+    std::bitset<64> shownColumns = m_fieldsGrid->GetShownColumns();
 
     if( shownColumns != m_shownColumns )
     {
@@ -1177,21 +1079,6 @@ void DIALOG_SYMBOL_PROPERTIES::OnUpdateUI( wxUpdateUIEvent& event )
         if( !m_fieldsGrid->IsCellEditControlShown() )
             AdjustFieldsGridColumns();
     }
-
-#ifdef KICAD_SPICE
-    wxString simEnable;
-
-    for( int ii = MANDATORY_FIELDS; ii < m_fieldsGrid->GetNumberRows(); ++ii )
-    {
-        if( m_fieldsGrid->GetCellValue( ii, FDC_NAME ) == SIM_ENABLE_FIELD )
-        {
-            simEnable = m_fieldsGrid->GetCellValue( ii, FDC_VALUE );
-            break;
-        }
-    }
-
-    m_cbExcludeFromSim->SetValue( simEnable == wxS( "0" ) );
-#endif
 }
 
 
@@ -1229,7 +1116,7 @@ void DIALOG_SYMBOL_PROPERTIES::OnSizeFieldsGrid( wxSizeEvent& event )
 {
     wxSize new_size = event.GetSize();
 
-    if( ( !m_editorShown || m_lastRequestedSize != new_size ) && m_fieldsSize != new_size )
+    if( ( !m_editorShown || m_lastRequestedFieldsSize != new_size ) && m_fieldsSize != new_size )
     {
         m_fieldsSize = new_size;
 
@@ -1237,9 +1124,9 @@ void DIALOG_SYMBOL_PROPERTIES::OnSizeFieldsGrid( wxSizeEvent& event )
     }
 
     // We store this value to check whether the dialog is changing size.  This might indicate
-    // that the user is scaling the dialog with an editor shown.  Some editors do not close
-    // (at least on GTK) when the user drags a dialog corner
-    m_lastRequestedSize = new_size;
+    // that the user is scaling the dialog with a grid-cell-editor shown.  Some editors do not
+    // close (at least on GTK) when the user drags a dialog corner
+    m_lastRequestedFieldsSize = new_size;
 
     // Always propagate for a grid repaint (needed if the height changes, as well as width)
     event.Skip();
@@ -1250,13 +1137,18 @@ void DIALOG_SYMBOL_PROPERTIES::OnSizePinsGrid( wxSizeEvent& event )
 {
     wxSize new_size = event.GetSize();
 
-    if( m_pinsSize != new_size )
+    if( ( !m_editorShown || m_lastRequestedPinsSize != new_size ) && m_pinsSize != new_size )
     {
         m_pinsSize = new_size;
 
         AdjustPinsGridColumns();
     }
 
+    // We store this value to check whether the dialog is changing size.  This might indicate
+    // that the user is scaling the dialog with a grid-cell-editor shown.  Some editors do not
+    // close (at least on GTK) when the user drags a dialog corner
+    m_lastRequestedPinsSize = new_size;
+
     // Always propagate for a grid repaint (needed if the height changes, as well as width)
     event.Skip();
 }
@@ -1327,3 +1219,12 @@ void DIALOG_SYMBOL_PROPERTIES::onUpdateEditLibrarySymbol( wxUpdateUIEvent& event
     event.Enable( m_symbol && m_symbol->GetLibSymbolRef() );
 }
 
+
+void DIALOG_SYMBOL_PROPERTIES::OnPageChanging( wxBookCtrlEvent& aEvent )
+{
+    if( !m_fieldsGrid->CommitPendingChanges() )
+        aEvent.Veto();
+
+    if( !m_pinGrid->CommitPendingChanges() )
+        aEvent.Veto();
+}
