@@ -434,8 +434,7 @@ SCH_EDIT_FRAME::SCH_EDIT_FRAME( KIWAY* aKiway, wxWindow* aParent ) :
 
     Bind( EDA_EVT_CLOSE_DIALOG_BOOK_REPORTER, &SCH_EDIT_FRAME::onCloseSymbolDiffDialog, this );
     Bind( EDA_EVT_CLOSE_ERC_DIALOG, &SCH_EDIT_FRAME::onCloseErcDialog, this );
-    Bind( EDA_EVT_CLOSE_DIALOG_SYMBOL_FIELDS_TABLE, &SCH_EDIT_FRAME::onCloseSymbolFieldsTableDialog,
-          this );
+    Bind( EDA_EVT_CLOSE_DIALOG_SYMBOL_FIELDS_TABLE, &SCH_EDIT_FRAME::onCloseSymbolFieldsTableDialog, this );
 }
 
 
@@ -1169,7 +1168,8 @@ void SCH_EDIT_FRAME::OnModify()
     if( GetScreen() )
         GetScreen()->SetContentModified();
 
-    m_autoSaveRequired = true;
+    if( m_isClosing )
+        return;
 
     if( GetCanvas() )
         GetCanvas()->Refresh();
@@ -1520,7 +1520,7 @@ void SCH_EDIT_FRAME::PrintPage( const RENDER_SETTINGS* aSettings )
 void SCH_EDIT_FRAME::RefreshOperatingPointDisplay()
 {
     SCHEMATIC_SETTINGS& settings = m_schematic->Settings();
-    SIM_LIB_MGR         simLibMgr( &Prj(), &Schematic() );
+    SIM_LIB_MGR         simLibMgr( &Prj() );
     NULL_REPORTER       devnull;
 
     // Patch for bug early in V7.99 dev
@@ -1611,8 +1611,13 @@ void SCH_EDIT_FRAME::RefreshOperatingPointDisplay()
             }
             else
             {
-                SIM_MODEL& model = simLibMgr.CreateModel( &GetCurrentSheet(), *symbol,
-                                                          devnull ).model;
+                std::vector<EMBEDDED_FILES*> embeddedFilesStack;
+                embeddedFilesStack.push_back( m_schematic->GetEmbeddedFiles() );
+                embeddedFilesStack.push_back( symbol->GetEmbeddedFiles() );
+
+                simLibMgr.SetFilesStack( embeddedFilesStack );
+
+                SIM_MODEL& model = simLibMgr.CreateModel( &GetCurrentSheet(), *symbol, devnull ).model;
 
                 SPICE_ITEM spiceItem;
                 spiceItem.refName = ref;
