@@ -272,7 +272,7 @@ DIALOG_PAD_PROPERTIES::DIALOG_PAD_PROPERTIES( PCB_BASE_FRAME* aParent, PAD* aPad
     if( m_padType->GetSelection() != PTH_DLG_TYPE && m_padType->GetSelection() != NPTH_DLG_TYPE )
     {
         m_gbSizerHole->Show( false );
-        m_staticline6->Show( false );
+        m_staticline71->Show( false );
     }
 
     // Now all widgets have the size fixed, call FinishDialogSettings
@@ -1122,7 +1122,7 @@ void DIALOG_PAD_PROPERTIES::PadTypeSelected( wxCommandEvent& event )
                          m_previewPad->GetKeepTopBottom() );
 
     m_gbSizerHole->Show( hasHole );
-    m_staticline6->Show( hasHole );
+    m_staticline71->Show( hasHole );
 
     if( !hasHole )
     {
@@ -1742,10 +1742,14 @@ bool DIALOG_PAD_PROPERTIES::transferDataToPad( PAD* aPad )
     else
         aPad->SetLocalSolderPasteMarginRatio( m_pasteMarginRatio.GetDoubleValue() / 100.0 );
 
-    if( !m_spokeWidth.IsNull() )
+    if( m_spokeWidth.IsNull() )
+        aPad->SetLocalThermalSpokeWidthOverride( {} );
+    else
         aPad->SetLocalThermalSpokeWidthOverride( m_spokeWidth.GetIntValue() );
 
-    if( !m_thermalGap.IsNull() )
+    if( m_thermalGap.IsNull() )
+        aPad->SetLocalThermalGapOverride( {} );
+    else
         aPad->SetLocalThermalGapOverride( m_thermalGap.GetIntValue() );
 
     aPad->SetThermalSpokeAngle( m_spokeAngle.GetAngleValue() );
