@@ -2,7 +2,7 @@
  * KiRouter - a push-and-(sometimes-)shove PCB router
  *
  * Copyright (C) 2013-2019 CERN
- * Copyright (C) 2016-2023 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright The KiCad Developers, see AUTHORS.txt for contributors.
  *
  * @author Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
  *
@@ -28,6 +28,7 @@
 
 #include <geometry/seg.h>
 #include <geometry/shape_line_chain.h>
+#include <zone.h>
 
 #include <wx/log.h>
 
@@ -48,7 +49,7 @@
 namespace PNS {
 
 #ifdef DEBUG
-static std::unordered_set<NODE*> allocNodes;
+static std::unordered_set<const NODE*> allocNodes;
 #endif
 
 NODE::NODE()
@@ -59,7 +60,6 @@ NODE::NODE()
     m_maxClearance = 800000;    // fixme: depends on how thick traces are.
     m_ruleResolver = nullptr;
     m_index = new INDEX;
-    m_collisionQueryScope = CQS_ALL_RULES;
 
 #ifdef DEBUG
     allocNodes.insert( this );
@@ -87,10 +87,36 @@ NODE::~NODE()
 
     m_joints.clear();
 
+    std::vector<const ITEM*> toDelete;
+
+    toDelete.reserve( m_index->Size() );
+
     for( ITEM* item : *m_index )
     {
-        if( item->BelongsTo( this ) )
-            delete item;
+        if( item->BelongsTo( this ) && item->OfKind( ITEM::HOLE_T ) )
+        {
+#ifdef DEBUG
+            HOLE* hole = static_cast<HOLE*>( item );
+
+            // If a hole is no longer owned by the same NODE as its parent then we're in a
+            // heap of trouble.
+            if( hole->ParentPadVia() && !hole->ParentPadVia()->BelongsTo( this ) )
+                assert( false );
+#endif
+
+            toDelete.push_back( item );
+        }
+    }
+
+    for( const ITEM* item : toDelete )
+    {
+        wxLogTrace( wxT( "PNS" ), wxT( "del item %p type %s" ), item, item->KindStr().c_str() );
+        delete item;
+    }
+
+    if( m_ruleResolver )
+    {
+        m_ruleResolver->ClearCacheForItems( toDelete );
     }
 
     releaseGarbage();
@@ -101,38 +127,16 @@ NODE::~NODE()
 
 
 int NODE::GetClearance( const ITEM* aA, const ITEM* aB, bool aUseClearanceEpsilon ) const
-{
-   if( !m_ruleResolver )
-        return 100000;
-
-   if( aA->IsVirtual() || aB->IsVirtual() )
-       return 0;
-
-   return m_ruleResolver->Clearance( aA, aB, aUseClearanceEpsilon );
-}
-
-
-int NODE::GetHoleClearance( const ITEM* aA, const ITEM* aB, bool aUseClearanceEpsilon ) const
 {
     if( !m_ruleResolver )
-        return 0;
+        return 100000;
 
     if( aA->IsVirtual() || aB->IsVirtual() )
         return 0;
 
-    return m_ruleResolver->HoleClearance( aA, aB, aUseClearanceEpsilon );
-}
+    int cl = m_ruleResolver->Clearance( aA, aB, aUseClearanceEpsilon );
 
-
-int NODE::GetHoleToHoleClearance( const ITEM* aA, const ITEM* aB, bool aUseClearanceEpsilon ) const
-{
-   if( !m_ruleResolver )
-        return 0;
-
-   if( aA->IsVirtual() || aB->IsVirtual() )
-       return 0;
-
-   return m_ruleResolver->HoleToHoleClearance( aA, aB, aUseClearanceEpsilon );
+    return cl;
 }
 
 
@@ -147,7 +151,6 @@ NODE* NODE::Branch()
     child->m_ruleResolver = m_ruleResolver;
     child->m_root = isRoot() ? this : m_root;
     child->m_maxClearance = m_maxClearance;
-    child->m_collisionQueryScope = m_collisionQueryScope;
 
     // Immediate offspring of the root branch needs not copy anything. For the rest, deep-copy
     // joints, overridden item maps and pointers to stored items.
@@ -211,23 +214,11 @@ bool OBSTACLE_VISITOR::visit( ITEM* aCandidate )
 // function object that visits potential obstacles and performs the actual collision refining
 struct NODE::DEFAULT_OBSTACLE_VISITOR : public OBSTACLE_VISITOR
 {
-    OBSTACLES& m_tab;
+    COLLISION_SEARCH_CONTEXT* m_ctx;
 
-    int        m_kindMask;          ///<  (solids, vias, segments, etc...)
-    int        m_limitCount;
-    int        m_matchCount;
-    bool       m_differentNetsOnly;
-    int        m_overrideClearance;
-
-    DEFAULT_OBSTACLE_VISITOR( NODE::OBSTACLES& aTab, const ITEM* aItem, int aKindMask,
-                              bool aDifferentNetsOnly, int aOverrideClearance ) :
+    DEFAULT_OBSTACLE_VISITOR( COLLISION_SEARCH_CONTEXT* aCtx, const ITEM* aItem ) :
         OBSTACLE_VISITOR( aItem ),
-        m_tab( aTab ),
-        m_kindMask( aKindMask ),
-        m_limitCount( -1 ),
-        m_matchCount( 0 ),
-        m_differentNetsOnly( aDifferentNetsOnly ),
-        m_overrideClearance( aOverrideClearance )
+        m_ctx( aCtx )
     {
     }
 
@@ -235,33 +226,25 @@ struct NODE::DEFAULT_OBSTACLE_VISITOR : public OBSTACLE_VISITOR
     {
     }
 
-    void SetCountLimit( int aLimit )
-    {
-        m_limitCount = aLimit;
-    }
-
     bool operator()( ITEM* aCandidate ) override
     {
-        if( !aCandidate->OfKind( m_kindMask ) )
+        if( !aCandidate->OfKind( m_ctx->options.m_kindMask ) )
+            return true;
+
+        // Collisions with self aren't a thing; don't spend time on them.
+        if( m_item == aCandidate )
+            return true;
+
+        if( m_ctx->options.m_filter && !m_ctx->options.m_filter( aCandidate ) )
             return true;
 
         if( visit( aCandidate ) )
             return true;
 
-        if( !aCandidate->Collide( m_item, m_node, m_differentNetsOnly, m_overrideClearance ) )
+        if( !aCandidate->Collide( m_item, m_node, m_layerContext.value_or( -1 ), m_ctx ) )
             return true;
 
-        OBSTACLE obs;
-
-        obs.m_item = aCandidate;
-        obs.m_head = m_item;
-        obs.m_distFirst = INT_MAX;
-        obs.m_maxFanoutWidth = 0;
-        m_tab.push_back( obs );
-
-        m_matchCount++;
-
-        if( m_limitCount > 0 && m_matchCount >= m_limitCount )
+        if( m_ctx->options.m_limitCount > 0 && m_ctx->obstacles.size() >= m_ctx->options.m_limitCount )
             return false;
 
         return true;
@@ -269,28 +252,28 @@ struct NODE::DEFAULT_OBSTACLE_VISITOR : public OBSTACLE_VISITOR
 };
 
 
-int NODE::QueryColliding( const ITEM* aItem, NODE::OBSTACLES& aObstacles, int aKindMask,
-                          int aLimitCount, bool aDifferentNetsOnly, int aOverrideClearance )
+int NODE::QueryColliding( const ITEM* aItem, NODE::OBSTACLES& aObstacles,
+                          const COLLISION_SEARCH_OPTIONS& aOpts ) const
 {
+    COLLISION_SEARCH_CONTEXT ctx( aObstacles, aOpts );
+
     /// By default, virtual items cannot collide
     if( aItem->IsVirtual() )
         return 0;
 
-    DEFAULT_OBSTACLE_VISITOR visitor( aObstacles, aItem, aKindMask, aDifferentNetsOnly,
-                                      aOverrideClearance );
+    DEFAULT_OBSTACLE_VISITOR visitor( &ctx, aItem );
 
 #ifdef DEBUG
     assert( allocNodes.find( this ) != allocNodes.end() );
 #endif
 
-    visitor.SetCountLimit( aLimitCount );
     visitor.SetWorld( this, nullptr );
 
     // first, look for colliding items in the local index
     m_index->Query( aItem, m_maxClearance, visitor );
 
     // if we haven't found enough items, look in the root branch as well.
-    if( !isRoot() && ( visitor.m_matchCount < aLimitCount || aLimitCount < 0 ) )
+    if( !isRoot() && ( ctx.obstacles.size() < aOpts.m_limitCount || aOpts.m_limitCount < 0 ) )
     {
         visitor.SetWorld( m_root, this );
         m_root->m_index->Query( aItem, m_maxClearance, visitor );
@@ -300,12 +283,12 @@ int NODE::QueryColliding( const ITEM* aItem, NODE::OBSTACLES& aObstacles, int aK
 }
 
 
-NODE::OPT_OBSTACLE NODE::NearestObstacle( const LINE* aLine, int aKindMask,
-                                          const std::set<ITEM*>* aRestrictedSet,
-                                          bool aUseClearanceEpsilon )
+NODE::OPT_OBSTACLE NODE::NearestObstacle( const LINE* aLine,
+                                          const COLLISION_SEARCH_OPTIONS& aOpts )
 {
-    OBSTACLES obstacleList;
-    obstacleList.reserve( 100 );
+    DIRECTION_45::CORNER_MODE cornerMode = ROUTER::GetInstance()->Settings().GetCornerMode();
+    const int                 clearanceEpsilon = GetRuleResolver()->ClearanceEpsilon();
+    OBSTACLES                 obstacleList;
 
     for( int i = 0; i < aLine->CLine().SegmentCount(); i++ )
     {
@@ -314,11 +297,11 @@ NODE::OPT_OBSTACLE NODE::NearestObstacle( const LINE* aLine, int aKindMask,
         // Disabling the cache will lead to slowness.
 
         const SEGMENT s( *aLine, aLine->CLine().CSegment( i ) );
-        QueryColliding( &s, obstacleList, aKindMask );
+        QueryColliding( &s, obstacleList, aOpts );
     }
 
     if( aLine->EndsWithVia() )
-        QueryColliding( &aLine->Via(), obstacleList, aKindMask );
+        QueryColliding( &aLine->Via(), obstacleList, aOpts );
 
     if( obstacleList.empty() )
         return OPT_OBSTACLE();
@@ -330,20 +313,15 @@ NODE::OPT_OBSTACLE NODE::NearestObstacle( const LINE* aLine, int aKindMask,
     nearest.m_maxFanoutWidth = 0;
 
     auto updateNearest =
-            [&]( const SHAPE_LINE_CHAIN::INTERSECTION& pt, ITEM* obstacle,
-                 const SHAPE_LINE_CHAIN& hull, bool isHole )
+            [&]( const SHAPE_LINE_CHAIN::INTERSECTION& pt, const OBSTACLE& obstacle )
             {
                 int dist = aLine->CLine().PathLength( pt.p, pt.index_their );
 
                 if( dist < nearest.m_distFirst )
                 {
+                    nearest = obstacle;
                     nearest.m_distFirst = dist;
                     nearest.m_ipFirst = pt.p;
-                    nearest.m_item = obstacle;
-                    nearest.m_hull = hull;
-
-                    obstacle->Mark( isHole ? obstacle->Marker() | MK_HOLE
-                                           : obstacle->Marker() & ~MK_HOLE );
                 }
             };
 
@@ -354,13 +332,20 @@ NODE::OPT_OBSTACLE NODE::NearestObstacle( const LINE* aLine, int aKindMask,
 
     for( const OBSTACLE& obstacle : obstacleList )
     {
-        if( aRestrictedSet && aRestrictedSet->find( obstacle.m_item ) == aRestrictedSet->end() )
-            continue;
-
-        int clearance =
-            GetClearance( obstacle.m_item, aLine, aUseClearanceEpsilon ) + aLine->Width() / 2;
+        int clearance = GetClearance( obstacle.m_item, aLine, aOpts.m_useClearanceEpsilon )
+                            + aLine->Width() / 2;
 
         obstacleHull = obstacle.m_item->Hull( clearance, 0, layer );
+
+        if( cornerMode == DIRECTION_45::MITERED_90 || cornerMode == DIRECTION_45::ROUNDED_90 )
+        {
+            BOX2I bbox = obstacleHull.BBox();
+            obstacleHull.Clear();
+            obstacleHull.Append( bbox.GetLeft(),  bbox.GetTop()    );
+            obstacleHull.Append( bbox.GetRight(), bbox.GetTop()    );
+            obstacleHull.Append( bbox.GetRight(), bbox.GetBottom() );
+            obstacleHull.Append( bbox.GetLeft(),  bbox.GetBottom() );
+        }
         //debugDecorator->AddLine( obstacleHull, 2, 40000, "obstacle-hull-test" );
         //debugDecorator->AddLine( aLine->CLine(), 5, 40000, "obstacle-test-line" );
 
@@ -371,88 +356,38 @@ NODE::OPT_OBSTACLE NODE::NearestObstacle( const LINE* aLine, int aKindMask,
         {
             //debugDecorator->AddPoint( ip.p, ip.valid?3:6, 100000, (const char *) wxString::Format("obstacle-isect-point-%d" ).c_str() );
             if( ip.valid )
-                updateNearest( ip, obstacle.m_item, obstacleHull, false );
+                updateNearest( ip, obstacle );
         }
 
         if( aLine->EndsWithVia() )
         {
             const VIA& via = aLine->Via();
-            // Don't use via.Drill(); it doesn't include the plating thickness
-
-            int viaHoleRadius = static_cast<const SHAPE_CIRCLE*>( via.Hole() )->GetRadius();
-
-            int viaClearance = GetClearance( obstacle.m_item, &via, aUseClearanceEpsilon )
-                               + via.Diameter() / 2;
-            int holeClearance =
-                    GetHoleClearance( obstacle.m_item, &via, aUseClearanceEpsilon ) + viaHoleRadius;
-
-            if( holeClearance > viaClearance )
-                viaClearance = holeClearance;
+            int viaClearance = GetClearance( obstacle.m_item, &via, aOpts.m_useClearanceEpsilon )
+                               + via.Diameter( aLine->Layer() ) / 2;
 
             obstacleHull = obstacle.m_item->Hull( viaClearance, 0, layer );
+
+            if( cornerMode == DIRECTION_45::MITERED_90 || cornerMode == DIRECTION_45::ROUNDED_90 )
+            {
+                BOX2I bbox = obstacleHull.BBox();
+                obstacleHull.Clear();
+                obstacleHull.Append( bbox.GetLeft(),  bbox.GetTop()    );
+                obstacleHull.Append( bbox.GetRight(), bbox.GetTop()    );
+                obstacleHull.Append( bbox.GetRight(), bbox.GetBottom() );
+                obstacleHull.Append( bbox.GetLeft(),  bbox.GetBottom() );
+            }
             //debugDecorator->AddLine( obstacleHull, 3 );
 
             intersectingPts.clear();
             HullIntersection( obstacleHull, aLine->CLine(), intersectingPts );
 
-            // obstacleHull.Intersect( aLine->CLine(), intersectingPts, true );
-
             for( const SHAPE_LINE_CHAIN::INTERSECTION& ip : intersectingPts )
-                updateNearest( ip, obstacle.m_item, obstacleHull, false );
-        }
-
-        if( ( m_collisionQueryScope == CQS_ALL_RULES
-              || !ROUTER::GetInstance()->GetInterface()->IsFlashedOnLayer( obstacle.m_item,
-                                                                           layer ) )
-            && obstacle.m_item->Hole() )
-        {
-            clearance = GetHoleClearance( obstacle.m_item, aLine, aUseClearanceEpsilon );
-            int copperClearance = GetClearance( obstacle.m_item, aLine, aUseClearanceEpsilon );
-
-            clearance = std::max( clearance, copperClearance );
-
-            obstacleHull = obstacle.m_item->HoleHull( clearance, aLine->Width(), layer );
-
-            intersectingPts.clear();
-            HullIntersection( obstacleHull, aLine->CLine(), intersectingPts );
-
-            for( const SHAPE_LINE_CHAIN::INTERSECTION& ip : intersectingPts )
-                updateNearest( ip, obstacle.m_item, obstacleHull, true );
-
-            if( aLine->EndsWithVia() )
-            {
-                const VIA& via = aLine->Via();
-                // Don't use via.Drill(); it doesn't include the plating thickness
-                int viaHoleRadius = static_cast<const SHAPE_CIRCLE*>( via.Hole() )->GetRadius();
-
-                int viaClearance = GetClearance( obstacle.m_item, &via, aUseClearanceEpsilon )
-                                   + via.Diameter() / 2;
-                int holeClearance = GetHoleClearance( obstacle.m_item, &via, aUseClearanceEpsilon )
-                                    + viaHoleRadius;
-                int holeToHole =
-                        GetHoleToHoleClearance( obstacle.m_item, &via, aUseClearanceEpsilon )
-                        + viaHoleRadius;
-
-                if( holeClearance > viaClearance )
-                    viaClearance = holeClearance;
-
-                if( holeToHole > viaClearance )
-                    viaClearance = holeToHole;
-
-                obstacleHull = obstacle.m_item->Hull( viaClearance, 0, layer );
-                //debugDecorator->AddLine( obstacleHull, 5 );
-
-                intersectingPts.clear();
-                HullIntersection( obstacleHull, aLine->CLine(), intersectingPts );
-
-                for( const SHAPE_LINE_CHAIN::INTERSECTION& ip : intersectingPts )
-                    updateNearest( ip, obstacle.m_item, obstacleHull, true );
-            }
+                updateNearest( ip, obstacle );
         }
     }
 
     if( nearest.m_distFirst == INT_MAX )
-        nearest.m_item = obstacleList[0].m_item;
+        nearest = (*obstacleList.begin());
 
     return nearest;
 }
@@ -465,7 +400,7 @@ NODE::OPT_OBSTACLE NODE::CheckColliding( const ITEM_SET& aSet, int aKindMask )
         OPT_OBSTACLE obs = CheckColliding( item, aKindMask );
 
         if( obs )
-            return  obs;
+            return obs;
     }
 
     return OPT_OBSTACLE();
@@ -474,9 +409,17 @@ NODE::OPT_OBSTACLE NODE::CheckColliding( const ITEM_SET& aSet, int aKindMask )
 
 NODE::OPT_OBSTACLE NODE::CheckColliding( const ITEM* aItemA, int aKindMask )
 {
-    OBSTACLES obs;
+    COLLISION_SEARCH_OPTIONS opts;
 
-    obs.reserve( 100 );
+    opts.m_kindMask = aKindMask;
+    opts.m_limitCount = 1;
+
+    return CheckColliding( aItemA, opts );
+}
+
+NODE::OPT_OBSTACLE NODE::CheckColliding( const ITEM* aItemA, const COLLISION_SEARCH_OPTIONS& aOpts )
+{
+    OBSTACLES obs;
 
     if( aItemA->Kind() == ITEM::LINE_T )
     {
@@ -491,23 +434,23 @@ NODE::OPT_OBSTACLE NODE::CheckColliding( const ITEM* aItemA, int aKindMask )
             // Disabling the cache will lead to slowness.
 
             const SEGMENT s( *line, l.CSegment( i ) );
-            n += QueryColliding( &s, obs, aKindMask, 1 );
+            n += QueryColliding( &s, obs, aOpts );
 
             if( n )
-                return OPT_OBSTACLE( obs[0] );
+                return OPT_OBSTACLE( *obs.begin() );
         }
 
         if( line->EndsWithVia() )
         {
-            n += QueryColliding( &line->Via(), obs, aKindMask, 1 );
+            n += QueryColliding( &line->Via(), obs, aOpts );
 
             if( n )
-                return OPT_OBSTACLE( obs[0] );
+                return OPT_OBSTACLE( *obs.begin() );
         }
     }
-    else if( QueryColliding( aItemA, obs, aKindMask, 1 ) > 0 )
+    else if( QueryColliding( aItemA, obs, aOpts ) > 0 )
     {
-        return OPT_OBSTACLE( obs[0] );
+        return OPT_OBSTACLE( *obs.begin() );
     }
 
     return OPT_OBSTACLE();
@@ -535,7 +478,8 @@ struct HIT_VISITOR : public OBSTACLE_VISITOR
 
         int cl = 0;
 
-        if( aItem->Shape()->Collide( &cp, cl ) )
+        // TODO(JE) padstacks -- this may not work
+        if( aItem->Shape( -1 )->Collide( &cp, cl ) )
             m_items.Add( aItem );
 
         return true;
@@ -574,9 +518,16 @@ const ITEM_SET NODE::HitTest( const VECTOR2I& aPoint ) const
 
 void NODE::addSolid( SOLID* aSolid )
 {
+    if( aSolid->HasHole() )
+    {
+        assert( aSolid->Hole()->BelongsTo( aSolid ) );
+        addHole( aSolid->Hole() );
+    }
+
     if( aSolid->IsRoutable() )
         linkJoint( aSolid->Pos(), aSolid->Layers(), aSolid->Net(), aSolid );
 
+    aSolid->SetOwner( this );
     m_index->Add( aSolid );
 }
 
@@ -590,19 +541,63 @@ void NODE::Add( std::unique_ptr< SOLID > aSolid )
 
 void NODE::addVia( VIA* aVia )
 {
+    if( aVia->HasHole() )
+    {
+        if( ! aVia->Hole()->BelongsTo( aVia ) )
+        {
+            assert( false );
+        }
+        addHole( aVia->Hole() );
+    }
+
     linkJoint( aVia->Pos(), aVia->Layers(), aVia->Net(), aVia );
+    aVia->SetOwner( this );
 
     m_index->Add( aVia );
 }
 
 
+void NODE::addHole( HOLE* aHole )
+{
+    // do we need holes in the connection graph?
+    //linkJoint( aHole->Pos(), aHole->Layers(), aHole->Net(), aHole );
+
+    aHole->SetOwner( this );
+    m_index->Add( aHole );
+}
+
+
 void NODE::Add( std::unique_ptr< VIA > aVia )
 {
-    aVia->SetOwner( this );
     addVia( aVia.release() );
 }
 
 
+void NODE::add( ITEM* aItem, bool aAllowRedundant )
+{
+    switch( aItem->Kind() )
+    {
+    case ITEM::ARC_T:
+        addArc( static_cast<ARC*>( aItem ) );
+        break;
+    case ITEM::SEGMENT_T:
+        addSegment( static_cast<SEGMENT*>( aItem ) );
+        break;
+    case ITEM::VIA_T:
+        addVia( static_cast<VIA*>( aItem ) );
+        break;
+    case ITEM::SOLID_T:
+        addSolid( static_cast<SOLID*>( aItem ) );
+        break;
+    case ITEM::HOLE_T:
+        // added by parent VIA_T or SOLID_T (pad)
+        break;
+    default:
+        assert( false );
+    }
+}
+
+
 void NODE::Add( LINE& aLine, bool aAllowRedundant )
 {
     assert( !aLine.IsLinked() );
@@ -642,7 +637,8 @@ void NODE::Add( LINE& aLine, bool aAllowRedundant )
                                                                    aLine.Net() ) ) )
             {
                 // another line could be referencing this segment too :(
-                aLine.Link( rseg );
+                if( !aLine.ContainsLink( rseg ) )
+                    aLine.Link( rseg );
             }
             else
             {
@@ -657,6 +653,8 @@ void NODE::Add( LINE& aLine, bool aAllowRedundant )
 
 void NODE::addSegment( SEGMENT* aSeg )
 {
+    aSeg->SetOwner( this );
+
     linkJoint( aSeg->Seg().A, aSeg->Layers(), aSeg->Net(), aSeg );
     linkJoint( aSeg->Seg().B, aSeg->Layers(), aSeg->Net(), aSeg );
 
@@ -676,7 +674,6 @@ bool NODE::Add( std::unique_ptr< SEGMENT > aSegment, bool aAllowRedundant )
     if( !aAllowRedundant && findRedundantSegment( aSegment.get() ) )
         return false;
 
-    aSegment->SetOwner( this );
     addSegment( aSegment.release() );
 
     return true;
@@ -685,6 +682,8 @@ bool NODE::Add( std::unique_ptr< SEGMENT > aSegment, bool aAllowRedundant )
 
 void NODE::addArc( ARC* aArc )
 {
+    aArc->SetOwner( this );
+
     linkJoint( aArc->Anchor( 0 ), aArc->Layers(), aArc->Net(), aArc );
     linkJoint( aArc->Anchor( 1 ), aArc->Layers(), aArc->Net(), aArc );
 
@@ -702,41 +701,11 @@ bool NODE::Add( std::unique_ptr< ARC > aArc, bool aAllowRedundant )
         return false;
     }
 
-    aArc->SetOwner( this );
     addArc( aArc.release() );
     return true;
 }
 
 
-void NODE::Add( std::unique_ptr< ITEM > aItem, bool aAllowRedundant )
-{
-    switch( aItem->Kind() )
-    {
-    case ITEM::SOLID_T:   Add( ItemCast<SOLID>( std::move( aItem ) ) );                    break;
-    case ITEM::SEGMENT_T: Add( ItemCast<SEGMENT>( std::move( aItem ) ), aAllowRedundant ); break;
-    case ITEM::VIA_T:     Add( ItemCast<VIA>( std::move( aItem ) ) );                      break;
-
-    case ITEM::ARC_T:
-        //todo(snh): Add redundant search
-        Add( ItemCast<ARC>( std::move( aItem ) ) );
-        break;
-
-    case ITEM::LINE_T:
-    {
-        // fixme(twl): I don't like unique_ptr in this methods as the router has its own garbage
-        // collecting mechanism. This particular case is used exclusively in
-        // ROUTER::GetUpdatedItems() for dumping debug logs. Please don't call Add ( up<LINE> )
-        // otherwise, dragons live here.
-        LINE *l = static_cast<LINE*>( aItem.get() );
-        Add( *l );
-        break;
-    }
-    default:
-        assert( false );
-    }
-}
-
-
 void NODE::AddEdgeExclusion( std::unique_ptr<SHAPE> aShape )
 {
     m_edgeExclusions.push_back( std::move( aShape ) );
@@ -757,21 +726,47 @@ bool NODE::QueryEdgeExclusions( const VECTOR2I& aPos ) const
 
 void NODE::doRemove( ITEM* aItem )
 {
+    bool holeRemoved = false; // fixme: better logic, I don't like this
+
     // case 1: removing an item that is stored in the root node from any branch:
     // mark it as overridden, but do not remove
     if( aItem->BelongsTo( m_root ) && !isRoot() )
+    {
         m_override.insert( aItem );
 
+        if( aItem->HasHole() )
+            m_override.insert( aItem->Hole() );
+    }
+
     // case 2: the item belongs to this branch or a parent, non-root branch,
     // or the root itself and we are the root: remove from the index
     else if( !aItem->BelongsTo( m_root ) || isRoot() )
+    {
         m_index->Remove( aItem );
 
+        if( aItem->HasHole() )
+        {
+            m_index->Remove( aItem->Hole() );
+            holeRemoved = true;
+        }
+    }
+
     // the item belongs to this particular branch: un-reference it
     if( aItem->BelongsTo( this ) )
     {
         aItem->SetOwner( nullptr );
         m_root->m_garbageItems.insert( aItem );
+        HOLE *hole = aItem->Hole();
+
+        if( hole )
+        {
+            if( ! holeRemoved )
+            {
+                m_index->Remove( hole ); // hole is not directly owned by NODE but by the parent SOLID/VIA.
+            }
+
+            hole->SetOwner( aItem );
+        }
     }
 }
 
@@ -790,15 +785,18 @@ void NODE::removeArcIndex( ARC* aArc )
 }
 
 
-void NODE::rebuildJoint( JOINT* aJoint, ITEM* aItem )
+void NODE::rebuildJoint( const JOINT* aJoint, const ITEM* aItem )
 {
+    if( !aJoint )
+        return;
+
     // We have to split a single joint (associated with a via or a pad, binding together multiple
     // layers) into multiple independent joints. As I'm a lazy bastard, I simply delete the
     // via/solid and all its links and re-insert them.
 
-    JOINT::LINKED_ITEMS links( aJoint->LinkList() );
-    JOINT::HASH_TAG tag;
-    int net = aItem->Net();
+    std::vector<ITEM*> links( aJoint->LinkList() );
+    JOINT::HASH_TAG    tag;
+    NET_HANDLE         net = aItem->Net();
 
     tag.net = net;
     tag.pos = aJoint->Pos();
@@ -826,18 +824,31 @@ void NODE::rebuildJoint( JOINT* aJoint, ITEM* aItem )
         }
     } while( split );
 
+    bool completelyErased = false;
+
+    if( !isRoot() && m_joints.find( tag ) == m_joints.end() )
+    {
+        JOINT jtDummy( tag.pos, PNS_LAYER_RANGE(-1), tag.net );
+
+        m_joints.insert( TagJointPair( tag, jtDummy ) );
+        completelyErased = true;
+    }
+
+
     // and re-link them, using the former via's link list
     for( ITEM* link : links )
     {
         if( link != aItem )
             linkJoint( tag.pos, link->Layers(), net, link );
+        else if( !completelyErased )
+            unlinkJoint( tag.pos, link->Layers(), net, link );
     }
 }
 
 
 void NODE::removeViaIndex( VIA* aVia )
 {
-    JOINT* jt = FindJoint( aVia->Pos(), aVia->Layers().Start(), aVia->Net() );
+    const JOINT* jt = FindJoint( aVia->Pos(), aVia->Layers().Start(), aVia->Net() );
     assert( jt );
     rebuildJoint( jt, aVia );
 }
@@ -849,7 +860,7 @@ void NODE::removeSolidIndex( SOLID* aSolid )
         return;
 
     // fixme: redundant code
-    JOINT* jt = FindJoint( aSolid->Pos(), aSolid->Layers().Start(), aSolid->Net() );
+    const JOINT* jt = FindJoint( aSolid->Pos(), aSolid->Layers().Start(), aSolid->Net() );
     assert( jt );
     rebuildJoint( jt, aSolid );
 }
@@ -858,14 +869,14 @@ void NODE::removeSolidIndex( SOLID* aSolid )
 void NODE::Replace( ITEM* aOldItem, std::unique_ptr< ITEM > aNewItem )
 {
     Remove( aOldItem );
-    Add( std::move( aNewItem ) );
+    add( aNewItem.release() );
 }
 
 
-void NODE::Replace( LINE& aOldLine, LINE& aNewLine )
+void NODE::Replace( LINE& aOldLine, LINE& aNewLine, bool aAllowRedundantSegments )
 {
     Remove( aOldLine );
-    Add( aNewLine );
+    Add( aNewLine, aAllowRedundantSegments );
 }
 
 
@@ -880,6 +891,11 @@ void NODE::Remove( VIA* aVia )
 {
     removeViaIndex( aVia );
     doRemove( aVia );
+
+    if( !aVia->Owner() )
+    {
+        assert( aVia->Hole()->BelongsTo( aVia ) );
+    }
 }
 
 
@@ -906,8 +922,18 @@ void NODE::Remove( ITEM* aItem )
         break;
 
     case ITEM::SOLID_T:
+    {
+        SOLID* solid = static_cast<SOLID*>( aItem );
+
+        if( solid->HasHole() )
+        {
+            Remove( solid->Hole() );
+            solid->Hole()->SetOwner( solid );
+        }
+
         Remove( static_cast<SOLID*>( aItem ) );
         break;
+    }
 
     case ITEM::SEGMENT_T:
         Remove( static_cast<SEGMENT*>( aItem ) );
@@ -924,8 +950,18 @@ void NODE::Remove( ITEM* aItem )
     }
 
     case ITEM::VIA_T:
+    {
+        VIA* via = static_cast<VIA*>( aItem );
+
+        if( via->HasHole() )
+        {
+            Remove( via->Hole() );
+            via->Hole()->SetOwner( via );
+        }
+
         Remove( static_cast<VIA*>( aItem ) );
         break;
+    }
 
     default:
         break;
@@ -944,6 +980,8 @@ void NODE::Remove( LINE& aLine )
             Remove( static_cast<SEGMENT*>( li ) );
         else if( li->OfKind( ITEM::ARC_T ) )
             Remove( static_cast<ARC*>( li ) );
+        else if( li->OfKind( ITEM::VIA_T ) )
+            Remove( static_cast<VIA*>( li ) );
     }
 
     aLine.SetOwner( nullptr );
@@ -964,7 +1002,8 @@ void NODE::followLine( LINKED_ITEM* aCurrent, bool aScanDirection, int& aPos, in
         const VECTOR2I p  = aCurrent->Anchor( aScanDirection ^ prevReversed );
         const JOINT*   jt = FindJoint( p, aCurrent );
 
-        assert( jt );
+        if( !jt )
+            break;
 
         aCorners[aPos]     = jt->Pos();
         aSegments[aPos]    = aCurrent;
@@ -974,7 +1013,9 @@ void NODE::followLine( LINKED_ITEM* aCurrent, bool aScanDirection, int& aPos, in
         {
             if( ( aScanDirection && jt->Pos() == aCurrent->Anchor( 0 ) )
                     || ( !aScanDirection && jt->Pos() == aCurrent->Anchor( 1 ) ) )
+            {
                 aArcReversed[aPos] = true;
+            }
         }
 
         aPos += ( aScanDirection ? 1 : -1 );
@@ -990,11 +1031,14 @@ void NODE::followLine( LINKED_ITEM* aCurrent, bool aScanDirection, int& aPos, in
 
         bool locked = aStopAtLockedJoints ? jt->IsLocked() : false;
 
-        if( locked || !jt->IsLineCorner( aFollowLockedSegments ) || aPos < 0 || aPos == aLimit )
+        if( locked || aPos < 0 || aPos == aLimit )
             break;
 
         aCurrent = jt->NextSegment( aCurrent, aFollowLockedSegments );
 
+        if( !aCurrent )
+            break;
+
         prevReversed = ( aCurrent && jt->Pos() == aCurrent->Anchor( aScanDirection ) );
     }
 }
@@ -1018,6 +1062,8 @@ const LINE NODE::AssembleLine( LINKED_ITEM* aSeg, int* aOriginSegmentIndex,
     pl.SetWidth( aSeg->Width() );
     pl.SetLayers( aSeg->Layers() );
     pl.SetNet( aSeg->Net() );
+    pl.SetParent( nullptr );
+    pl.SetSourceItem( aSeg->GetSourceItem() );
     pl.SetOwner( this );
 
     followLine( aSeg, false, i_start, MaxVerts, corners.data(), segs.data(), arcReversed.data(),
@@ -1049,7 +1095,7 @@ const LINE NODE::AssembleLine( LINKED_ITEM* aSeg, int* aOriginSegmentIndex,
             if( li->Kind() == ITEM::ARC_T )
             {
                 const ARC*       arc = static_cast<const ARC*>( li );
-                const SHAPE_ARC* sa  = static_cast<const SHAPE_ARC*>( arc->Shape() );
+                const SHAPE_ARC* sa  = static_cast<const SHAPE_ARC*>( arc->Shape( -1 ) );
 
                 int      nSegs     = line.PointCount();
                 VECTOR2I last      = nSegs ? line.CPoint( -1 ) : VECTOR2I();
@@ -1074,13 +1120,13 @@ const LINE NODE::AssembleLine( LINKED_ITEM* aSeg, int* aOriginSegmentIndex,
     }
 
     // Remove duplicate verts, but do NOT remove colinear segments here!
-    pl.Line().Simplify( false );
+    pl.Line().RemoveDuplicatePoints();
 
     // TODO: maintain actual segment index under simplification system
     if( aOriginSegmentIndex && *aOriginSegmentIndex >= pl.SegmentCount() )
         *aOriginSegmentIndex = pl.SegmentCount() - 1;
 
-    assert( pl.SegmentCount() != 0 );
+    wxASSERT_MSG( pl.SegmentCount() != 0, "assembled line should never be empty" );
 
     return pl;
 }
@@ -1129,7 +1175,7 @@ int NODE::FindLinesBetweenJoints( const JOINT& aA, const JOINT& aB, std::vector<
 
 void NODE::FixupVirtualVias()
 {
-    SEGMENT* locked_seg = nullptr;
+    const SEGMENT* locked_seg = nullptr;
     std::vector<VVIA*> vvias;
 
     for( auto& jointPair : m_joints )
@@ -1139,27 +1185,45 @@ void NODE::FixupVirtualVias()
         if( joint.Layers().IsMultilayer() )
             continue;
 
-        int  n_seg = 0, n_solid = 0, n_vias = 0;
-        int  prev_w          = -1;
-        int  max_w           = -1;
-        bool is_width_change = false;
-        bool is_locked       = false;
+        int                n_seg   = 0;
+        int                n_solid = 0;
+        int                n_vias  = 0;
+        int                prev_w    = -1;
+        bool               prev_mask = false;
+        std::optional<int> prev_mask_margin;
+        int                max_w           = -1;
+        bool               is_width_change = false;
+        bool               is_locked       = false;
 
-        for( const auto& lnk : joint.LinkList() )
+        for( const ITEM* item : joint.LinkList() )
         {
-            if( lnk.item->OfKind( ITEM::VIA_T ) )
+            if( item->OfKind( ITEM::VIA_T ) )
             {
                 n_vias++;
             }
-            else if( lnk.item->OfKind( ITEM::SOLID_T ) )
+            else if( item->OfKind( ITEM::SOLID_T ) )
             {
                 n_solid++;
             }
-            else if( const auto t = dyn_cast<PNS::SEGMENT*>( lnk.item ) )
+            else if( const auto t = dyn_cast<const PNS::SEGMENT*>( item ) )
             {
-                int w = t->Width();
+                int                w    = t->Width();
+                bool               mask = false;
+                std::optional<int> mask_margin;
 
-                if( prev_w >= 0 && w != prev_w )
+                if( PCB_TRACK* track = static_cast<PCB_TRACK*>( t->Parent() ) )
+                {
+                    mask = track->HasSolderMask();
+                    mask_margin = track->GetLocalSolderMaskMargin();
+                }
+
+                if( prev_w < 0 )
+                {
+                    prev_w = w;
+                    prev_mask = mask;
+                    prev_mask_margin = mask_margin;
+                }
+                else if( w != prev_w || mask != prev_mask || mask_margin != prev_mask_margin )
                 {
                     is_width_change = true;
                 }
@@ -1198,14 +1262,14 @@ void NODE::FixupVirtualVias()
 }
 
 
-JOINT* NODE::FindJoint( const VECTOR2I& aPos, int aLayer, int aNet )
+const JOINT* NODE::FindJoint( const VECTOR2I& aPos, int aLayer, NET_HANDLE aNet ) const
 {
     JOINT::HASH_TAG tag;
 
     tag.net = aNet;
     tag.pos = aPos;
 
-    JOINT_MAP::iterator f = m_joints.find( tag ), end = m_joints.end();
+    JOINT_MAP::const_iterator f = m_joints.find( tag ), end = m_joints.end();
 
     if( f == end && !isRoot() )
     {
@@ -1213,15 +1277,12 @@ JOINT* NODE::FindJoint( const VECTOR2I& aPos, int aLayer, int aNet )
         f = m_root->m_joints.find( tag );    // m_root->FindJoint(aPos, aLayer, aNet);
     }
 
-    if( f == end )
-        return nullptr;
-
     while( f != end )
     {
-        if( f->second.Layers().Overlaps( aLayer ) )
+        if( f->second.Pos() == aPos && f->second.Net() == aNet && f->second.Layers().Overlaps( aLayer ) )
             return &f->second;
 
-        ++f;
+        f++;
     }
 
     return nullptr;
@@ -1235,7 +1296,7 @@ void NODE::LockJoint( const VECTOR2I& aPos, const ITEM* aItem, bool aLock )
 }
 
 
-JOINT& NODE::touchJoint( const VECTOR2I& aPos, const LAYER_RANGE& aLayers, int aNet )
+JOINT& NODE::touchJoint( const VECTOR2I& aPos, const PNS_LAYER_RANGE& aLayers, NET_HANDLE aNet )
 {
     JOINT::HASH_TAG tag;
 
@@ -1296,7 +1357,8 @@ void JOINT::Dump() const
 }
 
 
-void NODE::linkJoint( const VECTOR2I& aPos, const LAYER_RANGE& aLayers, int aNet, ITEM* aWhere )
+void NODE::linkJoint( const VECTOR2I& aPos, const PNS_LAYER_RANGE& aLayers, NET_HANDLE aNet,
+                      ITEM* aWhere )
 {
     JOINT& jt = touchJoint( aPos, aLayers, aNet );
 
@@ -1304,7 +1366,8 @@ void NODE::linkJoint( const VECTOR2I& aPos, const LAYER_RANGE& aLayers, int aNet
 }
 
 
-void NODE::unlinkJoint( const VECTOR2I& aPos, const LAYER_RANGE& aLayers, int aNet, ITEM* aWhere )
+void NODE::unlinkJoint( const VECTOR2I& aPos, const PNS_LAYER_RANGE& aLayers, NET_HANDLE aNet,
+                        ITEM* aWhere )
 {
     // fixme: remove dangling joints
     JOINT& jt = touchJoint( aPos, aLayers, aNet );
@@ -1414,8 +1477,8 @@ void NODE::GetUpdatedItems( ITEM_VECTOR& aRemoved, ITEM_VECTOR& aAdded )
     for( ITEM* item : m_override )
         aRemoved.push_back( item );
 
-    for( INDEX::ITEM_SET::iterator i = m_index->begin(); i != m_index->end(); ++i )
-        aAdded.push_back( *i );
+    for( ITEM* item : *m_index )
+        aAdded.push_back( item );
 }
 
 
@@ -1437,6 +1500,9 @@ void NODE::releaseGarbage()
     if( !isRoot() )
         return;
 
+    std::vector<const ITEM*> cacheCheckItems;
+    cacheCheckItems.reserve( m_garbageItems.size() );
+
     for( ITEM* item : m_garbageItems )
     {
         if( !item->BelongsTo( this ) )
@@ -1444,6 +1510,11 @@ void NODE::releaseGarbage()
     }
 
     m_garbageItems.clear();
+
+    if( m_ruleResolver )
+    {
+        m_ruleResolver->ClearCacheForItems( cacheCheckItems );
+    }
 }
 
 
@@ -1457,9 +1528,14 @@ void NODE::Commit( NODE* aNode )
 
     for( ITEM* item : *aNode->m_index )
     {
+        if( item->HasHole() )
+        {
+            item->Hole()->SetOwner( item );
+        }
+
         item->SetRank( -1 );
         item->Unmark();
-        Add( std::unique_ptr<ITEM>( item ) );
+        add( item );
     }
 
     releaseChildren();
@@ -1473,7 +1549,7 @@ void NODE::KillChildren()
 }
 
 
-void NODE::AllItemsInNet( int aNet, std::set<ITEM*>& aItems, int aKindMask )
+void NODE::AllItemsInNet( NET_HANDLE aNet, std::set<ITEM*>& aItems, int aKindMask )
 {
     INDEX::NET_ITEMS_LIST* l_cur = m_index->GetItemsForNet( aNet );
 
@@ -1527,10 +1603,10 @@ void NODE::RemoveByMarker( int aMarker )
 }
 
 
-SEGMENT* NODE::findRedundantSegment( const VECTOR2I& A, const VECTOR2I& B, const LAYER_RANGE& lr,
-                                     int aNet )
+SEGMENT* NODE::findRedundantSegment( const VECTOR2I& A, const VECTOR2I& B, const PNS_LAYER_RANGE& lr,
+                                     NET_HANDLE aNet )
 {
-    JOINT* jtStart = FindJoint( A, lr.Start(), aNet );
+    const JOINT* jtStart = FindJoint( A, lr.Start(), aNet );
 
     if( !jtStart )
         return nullptr;
@@ -1562,10 +1638,10 @@ SEGMENT* NODE::findRedundantSegment( SEGMENT* aSeg )
 }
 
 
-ARC* NODE::findRedundantArc( const VECTOR2I& A, const VECTOR2I& B, const LAYER_RANGE& lr,
-                             int aNet )
+ARC* NODE::findRedundantArc( const VECTOR2I& A, const VECTOR2I& B, const PNS_LAYER_RANGE& lr,
+                             NET_HANDLE aNet )
 {
-    JOINT* jtStart = FindJoint( A, lr.Start(), aNet );
+    const JOINT* jtStart = FindJoint( A, lr.Start(), aNet );
 
     if( !jtStart )
         return nullptr;
@@ -1597,7 +1673,7 @@ ARC* NODE::findRedundantArc( ARC* aArc )
 }
 
 
-int NODE::QueryJoints( const BOX2I& aBox, std::vector<JOINT*>& aJoints, LAYER_RANGE aLayerMask,
+int NODE::QueryJoints( const BOX2I& aBox, std::vector<JOINT*>& aJoints, PNS_LAYER_RANGE aLayerMask,
                        int aKindMask )
 {
     int n = 0;
@@ -1637,11 +1713,10 @@ int NODE::QueryJoints( const BOX2I& aBox, std::vector<JOINT*>& aJoints, LAYER_RA
 
 ITEM *NODE::FindItemByParent( const BOARD_ITEM* aParent )
 {
-    if( aParent->IsConnected() )
+    if( aParent && aParent->IsConnected() )
     {
         const BOARD_CONNECTED_ITEM* cItem = static_cast<const BOARD_CONNECTED_ITEM*>( aParent );
-
-        INDEX::NET_ITEMS_LIST* l_cur = m_index->GetItemsForNet( cItem->GetNetCode() );
+        INDEX::NET_ITEMS_LIST*      l_cur = m_index->GetItemsForNet( cItem->GetNet() );
 
         if( l_cur )
         {
@@ -1656,4 +1731,39 @@ ITEM *NODE::FindItemByParent( const BOARD_ITEM* aParent )
     return nullptr;
 }
 
+
+std::vector<ITEM*> NODE::FindItemsByParent( const BOARD_ITEM* aParent )
+{
+    std::vector<ITEM*> ret;
+
+    for( ITEM* item : *m_index )
+    {
+        if( item->Parent() == aParent )
+            ret.push_back( item );
+    }
+
+    return ret;
 }
+
+
+VIA* NODE::FindViaByHandle ( const VIA_HANDLE& handle ) const
+{
+    const JOINT* jt = FindJoint( handle.pos, handle.layers.Start(), handle.net );
+
+    if( !jt )
+        return nullptr;
+
+    for( ITEM* item : jt->LinkList() )
+    {
+        if( item->OfKind( ITEM::VIA_T ) )
+        {
+            if( item->Net() == handle.net && item->Layers().Overlaps(handle.layers) )
+                return static_cast<VIA*>( item );
+        }
+    }
+
+    return nullptr;
+}
+
+}
+
